<?php 
/**
 * NOTICE OF LICENSE.
 *
 * This source module is subject to the following license: REGULAR LICENSE for one domain
 *
 * DISCLAIMER
 *
 * Not for sale.
 *
 * @author    Envía Ya S.A. de C.V
 * @copyright Envía Ya S.A. de C.V https://enviaya.com.mx
 * @license   REGULAR LICENSE
 */

class Enviaya_ApiRequestor
{
    /**
     * @var string $apiKey The API key that's to be used to make requests.
     */
    public $apiKey;
    private static $httpClient;
    
    public function __construct($apiKey = null)
    {
        $this->_apiKey = $apiKey;
    }
    
    /**
     * @param string $url The path to the API endpoint.
     *
     * @returns string The full path.
     */
    public static function apiUrl($url = '')
    {
        $apiBase = Enviaya::$apiBase;
        return "$apiBase$url";
    }
    
    private static function _encodeObjects($d)
    {
        if ($d instanceof Enviaya_ApiResource) {
            return Enviaya_Util::utf8($d->id);
        } else if ($d instanceof Enviaya_Object) {
            return Enviaya_Util::utf8($d->object_id);
        } else if ($d === true) {
            return true;
        } else if ($d === false) {
            return false;
        } else if (is_array($d)) {
            $res = array();
            foreach ($d as $k => $v)
                $res[$k] = self::_encodeObjects($v);
            return $res;
        } else {
            return Enviaya_Util::utf8($d);
        }
    }
    
    /**
     * @param string $method
     * @param string $url
     * @param array|null $params
     *
     * @return array An array whose first element is the response and second
     *    element is the API key used to make the request.
     */
    public function request($method, $url, $params = null)
    {
      
        if (!$params)
            $params = array();
        list($rbody, $rcode, $myApiKey) = $this->_requestRaw($method, $url, $params);
        $resp = $this->_interpretResponse($rbody, $rcode);
        return array(
            $resp,
            $myApiKey
        );
    }
    
    /**
     * @param string $rbody A JSON string.
     * @param int $rcode
     * @param array $resp
     *
     * @throws Enviaya_InvalidRequestError if the error is caused by the user.
     * @throws Enviaya_AuthenticationError if the error is caused by a lack of
     *    permissions.
     * @throws Enviaya_ApiError otherwise.
     */
    public function handleApiError($rbody, $rcode, $resp)
    {
        // Array is not currently being returned by API, making the below N/A 
        // if (!is_array($resp) || !isset($resp['error'])) {
        //   $msg = "Invalid response object from API: $rbody "
        //        ."(HTTP response code was $rcode)";
        //   throw new Enviaya_ApiError($msg, $rcode, $rbody, $resp);
        // }
        
        $msg = "message not set";
        $param = "parameters not set";
        $code = "code not set";
        
        // Temporary setting of msg to rbody
        $msg = $rbody;
        
     
        // Parameters necessary for error code construction are not provided
        // $error = $resp['error'];
        // $msg = isset($error['message']) ? $error['message'] : null;
        // $param = isset($error['param']) ? $error['param'] : null;
        // $code = isset($error['code']) ? $error['code'] : null;
        
        switch ($rcode) {
            case 400:
                throw new Enviaya_InvalidRequestError($msg, $param, $rcode, $rbody, $resp);
            case 404:
                throw new Enviaya_InvalidRequestError($msg, $param, $rcode, $rbody, $resp);
            case 401:
                throw new Enviaya_AuthenticationError($msg, $rcode, $rbody, $resp);
            
        }
    }

    public function getRequestHeaders()
    {

        $headers = array(
            'Content-Type: application/json',
            'Accept: application/json',
        );
        if (Enviaya::getApiVersion()){
            $headers[] = 'EnviaYa-API-Version: ' . Enviaya::getApiVersion();
        }

        return $headers;
    }
    
    private function _requestRaw($method, $url, $params)
    {

        $absUrl = $this->apiUrl($url);
        $params = self::_encodeObjects($params);
        $myApiKey = $this->_getApiKey();
        $headers = $this->getRequestHeaders();
       
        
        list($rbody, $rcode) = $this->httpClient()->request($method, $absUrl, $headers, $params);
        
        return array(
            $rbody,
            $rcode,
            $myApiKey
        );
    }
    
    private function _interpretResponse($rbody, $rcode)
    {
        
        try {
            $resp = json_decode($rbody, true);
            
        }
        catch (Exception $e) {
            $msg = "Invalid response body from API: $rbody " . "(HTTP response code was $rcode)";
            throw new Enviaya_ApiError($msg, $rcode, $rbody);
        }
       
        if ($rcode < 200 || $rcode >= 300) {
            $this->handleApiError($rbody, $rcode, $resp);
        }
        
        return $resp;
    }

    private function _getApiKey()
    {
        $apiKey = $this->_apiKey;
        if (!$apiKey)
            $apiKey = Enviaya::$apiKey;

        if (!$apiKey) {
            throw new Enviaya_AuthenticationError('No credentials provided.');
        }

        return $apiKey;
    }

    private function _getAuthorizationType($apiKey = '')
    {
        return strpos($apiKey, 'oauth.') === 0 ? 'Bearer' : 'EnviayaToken';
    }

    public static function setHttpClient($client)
    {
        self::$httpClient = $client;
    }

    public static function httpClient()
    {
        if (!self::$httpClient) {
            self::$httpClient = CurlClient::instance();
        }
        return self::$httpClient;
    }
}
