<?php 
/**
 * NOTICE OF LICENSE.
 *
 * This source module is subject to the following license: REGULAR LICENSE for one domain
 *
 * DISCLAIMER
 *
 * Not for sale.
 *
 * @author    Envía Ya S.A. de C.V
 * @copyright Envía Ya S.A. de C.V https://enviaya.com.mx
 * @license   REGULAR LICENSE
 */

abstract class Enviaya_ApiResource extends Enviaya_Object
{
    protected static function _scopedRetrieve($class, $id, $apiKey = null)
    {
        $instance = new $class($id, $apiKey);
        $instance->refresh();
        return $instance;
    }
    
    /**
     * @returns Enviaya_ApiResource The refreshed resource.
     */
    public function refresh()
    {
        $requestor = new Enviaya_ApiRequestor($this->_apiKey);
        $url = $this->instanceUrl();
        
        list($response, $apiKey) = $requestor->request('get', $url, $this->_retrieveOptions);
        $this->refreshFrom($response, $apiKey);
        return $this;
    }
    
    /**
     * @param string $class
     *
     * @returns string The name of the class, with namespacing and underscores
     *    stripped.
     */
    public static function className($class)
    {
        // Useful for namespaces: Foo\Enviaya_Shipment
        if ($postfixNamespaces = strrchr($class, '\\')) {
            $class = substr($postfixNamespaces, 1);
        }
        // Useful for underscored 'namespaces': Foo_Enviaya_Shipment
        if ($postfixFakeNamespaces = strrchr($class, 'Enviaya_')) {
            $class = $postfixFakeNamespaces;
        }
        if (substr($class, 0, strlen('Enviaya')) == 'Enviaya') {
            $class = substr($class, strlen('Enviaya'));
        }
        $class = str_replace('_', '', $class);
        $name = urlencode($class);
        $name = strtolower($name);
        return $name;
    }
    
    /**
     * @param string $class
     *
     * @returns string The endpoint URL for the given class.
     */
    public static function classUrl($class)
    {
        $base = self::_scopedLsb($class, 'className', $class);
        return "/v1/${base}s";
    }
    
    /**
     * @returns string The full API URL for this API resource.
     */
    public function instanceUrl()
    {
        $id = $this['id'];
        $class = get_class($this);
        if ($id === null) {
            $message = "Could not determine which URL to request: " . "$class instance has invalid ID: $id";
            throw new Enviaya_InvalidRequestError($message, null);
        }
        $id = Enviaya_Util::utf8($id);
        $base = $this->_lsb('classUrl', $class);
        $extn = urlencode($id);
        return "$base/$extn";
    }
    
    private static function _validateCall($method, $params = null, $apiKey = null)
    {
        if ($params && !is_array($params)) {
            $message = "You must pass an array as the first argument to Enviaya API ";
            throw new Enviaya_Error($message);
        }
        
        if ($apiKey && !is_string($apiKey)) {
            $message = 'The second argument to Enviaya API method calls is an ' . 'optional per-request apiKey (credentials base_64 encoded), which must be a string';
            throw new Enviaya_Error($message);
        }
    }
    
    protected static function _scopedAll($class, $params = null, $apiKey = null)
    {
        self::_validateCall('all', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class);
        list($response, $apiKey) = $requestor->request('get', $url, $params);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
    }
    
    protected static function _scopedCreate($class, $params = null, $apiKey = null)
    {
        self::_validateCall('create', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class);
        // Correction for malformed enviaya URLs
        $url = $url . "/";
        list($response, $apiKey) = $requestor->request('post', $url, $params);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
    }

    protected static function _scopedUpdate($class, $id, $params = null, $apiKey = null)
    {
        self::_validateCall('update', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class) . "/" . $id;
        list($response, $apiKey) = $requestor->request('put', $url, $params);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
    }

     // Special Case for Rates
    protected static function _scopedGetRates($class, $params = null, $apiKey = null)
    {
        self::_validateCall('create', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class);
        // Correction for malformed enviaya URLs
        $url = $url . "/";
        
        list($response, $apiKey) = $requestor->request('post', $url, $params); 
        
        if(is_array($response) && !isset($response['errors'])){
            $response_PS = array(
                'rates' => array(),
            );
           
            foreach($response as $key => $value){
               if(in_array($key,array('warning','subsidy'))){
                $response_PS[$key] = $value;
               }else{
                   if((string)$key != "store_pickup"){
                        foreach($value as $rate){
                            $response_PS['rates'][] = $rate;
                        }
                   }else{
                    $value['currency'] = $params['currency'];
                    $value['total_amount'] = 0.00;
                    $response_PS['rates'][] = $value;
                   }
                   
               }
            }
            
            $price = array();
            foreach ($response_PS['rates'] as $key => $row)
            {
                $price[$key] = $row['total_amount'];
            }
            array_multisort($price, SORT_ASC, $response_PS['rates']);

          
            $response = $response_PS;

           
        }
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
         
       
        
    }
    
    // Special Case for Rates which has parameters of the format: /url/parameter/url
    protected static function _scopedGet($class, $id, $params = null, $apiKey = null)
    {
        self::_validateCall('create', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = "/v1/shipments/" . $id . "/rates/{$params['currency']}";
        list($response, $apiKey) = $requestor->request('get', $url, $params);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
    }
    
    // Special case for Address Validation
    protected static function _scopedValidate($class, $id, $params = null, $apiKey = null)
    {
        self::_validateCall('create', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class) . "/" . $id . "/validate/";
        list($response, $apiKey) = $requestor->request('get', $url, $params);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
    }

    //Special case for Tracking Status
    protected static function _scopedGetStatus($class, $id, $params = null, $apiKey = null)
    {
        self::_validateCall('create', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class) . "/{$params['carrier']}/" . $id;
        list($response, $apiKey) = $requestor->request('get', $url, array());
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
    }
    
    protected static function _scopedAddBatch($class, $id, $params = null, $apiKey = null)
    {
        self::_validateCall('add', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class) . "/" . $id ."/" . 'add_shipments/';
        list($response, $apiKey) = $requestor->request('post', $url, $params);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);
    }

    protected static function _scopedRemoveBatch($class, $id, $params = null, $apiKey = null)
    {
        self::_validateCall('add', $params, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class) . "/" . $id ."/" .  'remove_shipments/';
        list($response, $apiKey) = $requestor->request('post', $url, $params);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);   
    }

    protected static function _scopedPurchaseBatch($class, $id, $apiKey = null)
    {
        self::_validateCall('purchase', null, $apiKey);
        $requestor = new Enviaya_ApiRequestor($apiKey);
        $url = self::_scopedLsb($class, 'classUrl', $class) . "/" . $id ."/" .  'purchase/';
        list($response, $apiKey) = $requestor->request('post', $url, null);
        return Enviaya_Util::convertToEnviayaObject($response, $apiKey);   
    }
}
